#include <stddef.h>
#include <qprocess.h>
#include <qstring.h>
#include <qvbox.h>
#include <qbutton.h>
#include <qmainwindow.h>
#include "graph.h"
#include "settings.h"
#include "device.h"
#include "export.h"
#include "report.h"

//class Tab : public QDialog
class Tab : public QMainWindow
{
  Q_OBJECT
public:
  Tab(const char* instr, QWidget *parent, long range, const char *fromInstr);
  void initSize(int w, int h);
  void sleep(const char *text);
  void wake();

  Graph *graph;
  Settings *settings;
  Command *command;
  Device *device;
  Export *exportXY;
  Report *report;
  QSplitter *split;
  QSplitter *rightSplit;
  QLabel *message;
  QVBox *foreignwarn;
  QPushButton *showparbutton;
  QLabel *warnlabel;
  
public slots:
  void leaveSettings();
  void gotoSettings();
  void handler();
  void openExport();
  void finishExport();
  void openReport();
  void leaveReport();
  void showPars();
  
signals:
  void restart();
  
private:
  int actFocus;
  int watchSocket;
  void resizeEvent(QResizeEvent *e);
  bool shown(QWidget *w);
  
  enum SplitNo {graph_split, settings_split, right_split, export_split, report_split, n_split};
  
  void checkSplit(SplitNo splitNo);
  bool splitChange(SplitNo splitNo);
  
  int widgetno;
  float oldSizes[2*3];
  SplitNo leftWidgetNo, rightWidgetNo;
  QValueList<int> sizes;
};
